package controllers

import (
	"hongze/hongze_api/models"
	"hongze/hongze_api/services"
	"hongze/hongze_api/utils"
	"time"
)

//用户
type UserController struct {
	BaseAuthController
}

//用户
type UserCommonController struct {
	BaseCommonController
}

// @Title 获取用户详情
// @Description 获取用户详情接口
// @Success 200 {object} models.UserDetail
// @router /detail [get]
func (this *UserController) Detail() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	user := this.User
	if user == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，用户信息为空"
		br.Ret = 408
		return
	}
	item, err := models.GetUserDetailByUserId(user.UserId)
	if err != nil {
		br.Msg = "获取信息失败"
		br.ErrMsg = "获取信息失败,Err:" + err.Error()
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = item
}

// @Title 获取短信验证码
// @Description 获取短信验证码接口
// @Param   Mobile   query   string  true       "手机号码"
// @Param   AreaNum   query   string  true       "地区编码"
// @Success 200 {object} models.UserDetail
// @router /getSmsCode [get]
func (this *UserCommonController) GetSmsCode() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	mobile := this.GetString("Mobile")
	areaNum := this.GetString("AreaNum")
	msgCode := utils.GetRandString(4)
	var result bool
	if areaNum == "86" || areaNum == "" || areaNum == "0" {
		result = services.SendSmsCode(mobile, msgCode)
	} else {
		result = services.SendSmsCodeGj(mobile, msgCode, areaNum)
	}
	//发送成功
	if result {
		item := new(models.MsgCode)
		item.OpenId = ""
		item.Code = msgCode
		item.Mobile = mobile
		item.ExpiredIn = time.Now().Unix() + 15*60
		item.Enabled = 1
		err := models.AddMsgCode(item)
		if err != nil {
			br.Msg = "发送失败"
			br.ErrMsg = "发送失败，Err:" + err.Error()
			return
		}
		br.Msg = "发送成功"
	} else {
		br.Msg = "发送失败"
	}
	br.Ret = 200
	br.Success = true
}

/*
$app->get('api/user/getSmsCode',"UserController@getSmsCode");//获取短信验证码
$app->post('api/user/checkSmsCode',"UserController@checkSmsCode");//校验短信验证码
$app->get('api/user/getEmailCode',"UserController@getEmailCode");//获取邮件验证码
$app->post('api/user/checkEmailCode',"UserController@checkEmailCode");//校验邮件验证码
$app->post('api/user/login',"UserController@login");//登录
$app->post('api/user/apply',"UserController@apply");//申请试用
$app->get('api/user/smallLimit',"UserController@smallLimit");//是否小套餐客户
*/
